#ifdef KITSCHY_DEBUG_MEMORY
#include "debug_memorymanager.h"
#endif

#ifdef _WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include "math.h"
#include "string.h"

#include "debug.h"

#include "GL/gl.h"
#include "SDL.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"

#include "BList.h"
#include "Symbol.h"
#include "GLTile.h"
#include "keyboardstate.h"

#include "debug.h"

#include "Binterface.h"


class CharacterBuffer {
public:
	CharacterBuffer(TTF_Font *f) {
		int i;
		for(i=0;i<256;i++) tile[i]=0;
		font = f;
	}
	~CharacterBuffer() {
		int i;
		for(i=0;i<256;i++) if (tile[i]!=0) delete tile[i];
	} //

	TTF_Font *font;
	GLTile *tile[256];
};

List<CharacterBuffer> text_tile_buffer;

void Binterface::clear_print_cache(void)
{
	text_tile_buffer.Delete();
} /* Binterface::clear_print_cache */



GLTile *get_text_tile(char c,TTF_Font *font)
{
	CharacterBuffer *cb,*selected = 0;

	text_tile_buffer.Rewind();
	while(text_tile_buffer.Iterate(cb)) {
		if (cb->font==font) {
			selected = cb;
		} // if
	} // if

	if (selected==0) {
		selected = new CharacterBuffer(font);
		text_tile_buffer.Add(selected);
	}

	if (selected->tile[c]==0) {
		char text[2];
		GLTile *tile;
		SDL_Surface *sfc;
		SDL_Color color;
		color.r=255;
		color.g=255;
		color.b=255;

		text[0]=c;
		text[1]=0;

		sfc=TTF_RenderText_Blended(font,text,color);
		tile=new GLTile(sfc);
		tile->set_smooth();
		selected->tile[c]=tile;
		return tile;
	} else {
		return selected->tile[c];
	}

} /* get_text_tile */


void Binterface::string_dimensions(char *text,TTF_Font *font,int *dx,int *dy)
{
	int i;
	int x=0,maxy=0;

	for(i=0;text[i]!=0;i++) {
		GLTile *t = get_text_tile(text[i],font);
		if (t!=0) {
			x+=t->get_dx();
			if (t->get_dy()>maxy) maxy = t->get_dy();
		}
	}

	*dx = x;
	*dy = maxy;
}


void Binterface::print_left(char *text,TTF_Font *font,float x,float y)
{
	int i;

	glNormal3f(0,0,1);
	for(i=0;text[i]!=0;i++) {
		GLTile *t = get_text_tile(text[i],font);
		if (t!=0) {
			t->set_hotspot(0,t->get_dy());
			t->draw(x,y,0,0,1);
			x+=t->get_dx();
		}
	}
} /* Binterface::print_left */


void Binterface::print_center(char *text,TTF_Font *font,float x,float y)
{
	int i,dx=0;

	for(i=0;text[i]!=0;i++) {
		GLTile *t = get_text_tile(text[i],font);
		if (t!=0) dx+=t->get_dx();
	}

	x-=dx/2;

	glNormal3f(0,0,1);
	for(i=0;text[i]!=0;i++) {
		GLTile *t = get_text_tile(text[i],font);
		if (t!=0) {
			t->set_hotspot(0,t->get_dy());
			t->draw(x,y,0,0,1);
			x+=t->get_dx();
		}
	}
} /* Binterface::print_center */


void Binterface::print_left(char *text,TTF_Font *font,float x,float y,float r,float g,float b,float a)
{
	int i;

	glNormal3f(0,0,1);
	for(i=0;text[i]!=0;i++) {
		GLTile *t = get_text_tile(text[i],font);
		if (t!=0) {
			t->set_hotspot(0,t->get_dy());
			t->draw(r,g,b,a,x,y,0,0,1);
			x+=t->get_dx();
		}

	}
} /* Binterface::print_left */


void Binterface::print_center(char *text,TTF_Font *font,float x,float y,float r,float g,float b,float a)
{
	int i,dx=0;

	for(i=0;text[i]!=0;i++) {
		GLTile *t = get_text_tile(text[i],font);
		if (t!=0) dx+=t->get_dx();
	}

	x-=dx/2;

	glNormal3f(0,0,1);
	for(i=0;text[i]!=0;i++) {
		GLTile *t = get_text_tile(text[i],font);
		if (t!=0) {
			t->set_hotspot(0,t->get_dy());
			t->draw(r,g,b,a,x,y,0,0,1);
			x+=t->get_dx();
		}
	}
} /* Binterface::print_center */
